#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <EEPROM.h>
#include <ESP8266HTTPClient.h>

#include "gpio.h"
#include "ext.h"
#define USE_SERIAL Serial

//WEB Server
ESP8266WebServer server;

//AP模式的SSID跟密碼
const char* softap_ssid = "PAW_Config";
const char* softap_pass = "123456789";

//從EEPROM讀出來的SSID跟密碼
String eeprom_ssid;
String eeprom_pass;
int eeprom_encrypt;

//IP位置
String AP_IP;
String STA_IP;
String D_UUID;

//UUID
uint8_t MAC_ADDR[6];
char UUID[18];

//reboot flag
int reboot_flag = 0;
int reboot_try = 0;
int status = WL_IDLE_STATUS;
bool is_epass = false;

//pin
int Config_btn = 14;		//Config Button

void setup() {
	Serial.begin(9600);
	while (!Serial){}
	
	is_epass = is_epassf();
	read_ep_wifi();						//讀取EEPROM的STA帳號密碼
	get_uuid();							//取得裝置UUID
	ESP.wdtDisable();
	ESP.wdtEnable(10000);
	
	Serial.printf("\r\n[BOOT] success.");
	pinMode(Config_btn, INPUT_PULLUP);
	
	if(digitalRead(Config_btn) == LOW) {
		//進入CONFIG MODE
		setupAP();
		while(1) {
			server.handleClient();
		}
	} else {
		WiFi.mode(WIFI_STA);
		set_sta_wifi();
	}
}

void loop() {
	Serial.printf("\r\nrun get data from url");
	HTTPClient http;
	http.begin("http://45.63.121.228/test.php");
	int httpCode = http.GET();
	if (httpCode > 0) {
		String payload = http.getString();
		Serial.print("\r\nbody: " + payload);
	}
	http.end();
	delay(5000);
}

/***************** 以下都是WIFI SOFTAP *****************/
bool is_epassf() {
	if(EEPROM.read(100) == 0x00){		//已經設定完成
		return true;
	}
	return false;
}

bool set_sta_wifi(void) {
	if(reboot_try > 10){
		ESP.reset();
	}
	if ( eeprom_ssid.length() > 1) {
		eeprom_ssid = eeprom_ssid.c_str();
		eeprom_pass = eeprom_pass.c_str();
		//eeprom_encrypt = eeprom_encrypt.c_int();
		WiFi.disconnect();
		if(eeprom_encrypt == 8){
			WiFi.begin(eeprom_ssid.c_str());
		}else{
			WiFi.begin(eeprom_ssid.c_str(), eeprom_pass.c_str());
		}
		int y = 0;
		while(y < 100){
			y++;
			server.handleClient();
			delay(100);
			if (WiFi.status() == WL_CONNECTED) {
				//Serial.print("\r\n[STA] connect success.");
				reboot_try = 0;
				reboot_flag = 0;
				return true;
			}
		}
	}
	reboot_try++;
	//Serial.printf("\r\n[STA] connect fail(%d).", reboot_try);
	return false;
}

void read_ep_wifi(void) {
	EEPROM.begin(512);
	delay(10);
	eeprom_ssid = "";
	for (int i = 0; i < 32; ++i) {
		if(EEPROM.read(i) == 0x00){
			break;
		}
		eeprom_ssid += char(EEPROM.read(i));
	}
	eeprom_encrypt = EEPROM.read(99);
	
	eeprom_pass = "";
	for (int i = 32; i < 96; ++i) {
		if(EEPROM.read(i) == 0x00){
			break;
		}
		eeprom_pass += char(EEPROM.read(i));
	}
	delay(10);
	eeprom_ssid.trim();
	eeprom_pass.trim();
}

void get_uuid(void) {
	// 取得裝置SSID
	WiFi.macAddress(MAC_ADDR);
	for (int i = 0; i < sizeof(MAC_ADDR); ++i) {
		sprintf(UUID, "%s%02X", UUID, MAC_ADDR[i]);
	}
}

void renew_ip_addr(void) {
	//AP
	IPAddress AP_IP_IA = WiFi.softAPIP();
	AP_IP = String(AP_IP_IA[0]) + '.' + String(AP_IP_IA[1]) + '.' + String(AP_IP_IA[2]) + '.' + String(AP_IP_IA[3]);
	if(AP_IP != "0.0.0.0") {
		Serial.printf("\r\n[AP] connected, ip = %s", AP_IP.c_str());
	} else {
		Serial.print("\r\n[AP] not connect.");
	}
	
	//STA
	IPAddress STA_IP_IA = WiFi.localIP();
	STA_IP = String(STA_IP_IA[0]) + '.' + String(STA_IP_IA[1]) + '.' + String(STA_IP_IA[2]) + '.' + String(STA_IP_IA[3]);
	if(STA_IP != "0.0.0.0") {
		Serial.printf("\r\n[STA] connected, ip = %s", STA_IP.c_str());
	} else {
		Serial.print("\r\n[STA] not connect.");
	}
}

void launchWeb(void) {
	renew_ip_addr();
	createWebServer();
	server.begin();
}

void setupAP(void) {
	WiFi.mode(WIFI_AP);
	//開啟AP模式 - Default:WPA2
	WiFi.softAP(softap_ssid, softap_pass);
	IPAddress ip = WiFi.localIP();
	server = ESP8266WebServer(ip, 80);
	//MDNS
	if (!MDNS.begin("config")) {
		Serial.println("Error setting up MDNS responder!");
		while(1) { 
			delay(1000);
		}
	}
	MDNS.setInstanceName("Config Mode");
	MDNS.addService("http", "tcp", 80);
	//WEB
	launchWeb();
}

void createWebServer(void) {
	server.on("/f.js", []() {
		server.send(200, "application/javascript", SCRIPT);
	});
	server.on("/wifi.json", []() {
		int n = WiFi.scanNetworks();
		String content = "";
		content += "[\r\n";
		for (int i = 0; i < n; ++i) {
			// Print SSID and RSSI for each network found
			content += "\t{\"ssid\": \"";
			content += WiFi.SSID(i);
			content += "\", \"rssi\": ";
			content += WiFi.RSSI(i);
			content += ", \"encrypt\": ";
			content += WiFi.encryptionType(i);
			if(i == n - 1) {
				content += "}\r\n";
			} else {
				content += "},\r\n";
			}
		}
		content += "]";
		server.send(200, "text/plain", content);
	});
	server.on("/setting.json", []() {
		//更新IP
		renew_ip_addr();
		WiFi.macAddress(MAC_ADDR);
		String content = "";
		content += "{\r\n";
		content += "\t\"uuid\": \"" + String(UUID) + "\",\r\n";
		content += "\t\"apip\": \"" + AP_IP + "\",\r\n";
		content += "\t\"ssid\": \"" + eeprom_ssid + "\",\r\n";
		content += "\t\"pass\": \"" + eeprom_pass + "\",\r\n";
		if(eeprom_encrypt == 8){
			content += "\t\"encrypt\": \"8\"\r\n";
		}else{
			content += "\t\"encrypt\": \"4\"\r\n";
		}
		content += "}";
		server.send(200, "text/plain", content);
	});
	server.on("/style.css", []() {
		server.send(200, "text/css", STYLE);
	});
	server.on("/", []() {
		server.send(200, "text/html", INDEX);
	});
	server.on("/setting", []() {
		server.send(200, "text/html", SAVE);
		//儲存到EEPROM
		String qsid = server.arg("ssid");
		String qpass = server.arg("pass");
		int qenpt = server.arg("wifi_encrypt").toInt();
		eeprom_ssid = qsid;
		eeprom_pass = qpass;
		eeprom_encrypt = qenpt;
		if (qsid.length() > 0) {
			EEPROM.write(100, 0x01);	//flag
			//儲存加密方式
			if(qenpt == 8){
				EEPROM.write(99, 0x08);	//wifi encrypt open
			}else{
				EEPROM.write(99, 0x04);	//wifi encrypt wpa2-aes
			}
			//SSID
			for (int i = 0; i < qsid.length(); ++i) {
				EEPROM.write(i, qsid[i]);
			}
			EEPROM.write(qsid.length(),0x00);
			//密碼
			for (int i = 0; i < qpass.length(); ++i) {
				EEPROM.write(32 + i, qpass[i]);
			}
			EEPROM.write(32 + qpass.length(),0x00);
			EEPROM.write(100, 0x00);	//flag
			EEPROM.commit();
			is_epass = true;
		}
		//Serial.printf("\r\n[WEB] save setting success.");
		reboot_flag = 1;
		//重新開機
		ESP.reset();
		set_sta_wifi();
	});
	server.on("/reboot", []() {
		server.send(200, "text/html", REBOOT);
		//重新開機
		ESP.reset();
	});
}
